Attribute VB_Name = "modUtil"
Option Explicit

Public Const MAX_PATH = 260

Public Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type

Public Type WIN32_FIND_DATA
        dwFileAttributes As Long
        ftCreationTime As FILETIME
        ftLastAccessTime As FILETIME
        ftLastWriteTime As FILETIME
        nFileSizeHigh As Long
        nFileSizeLow As Long
        dwReserved0 As Long
        dwReserved1 As Long
        cFileName As String * MAX_PATH
        cAlternate As String * 14
End Type
Public Const INVALID_HANDLE_VALUE = -1
Public Const ERROR_NO_MORE_FILES = 18&
Public Const ERROR_NO_NET_OR_BAD_PATH = 1203&

Private Const FORMAT_MESSAGE_FROM_SYSTEM = 4096
Private Const FORMAT_MESSAGE_IGNORE_INSERTS = 512

Public Declare Function FindFirstFile Lib "kernel32" Alias "FindFirstFileA" (ByVal lpFileName As String, lpFindFileData As WIN32_FIND_DATA) As Long
Public Declare Function FindClose Lib "kernel32" (ByVal hFindFile As Long) As Long
Public Declare Function GetDiskFreeSpaceEx Lib "kernel32" Alias "GetDiskFreeSpaceExA" (ByVal lpDirectoryName As String, lpFreeBytesAvailable As Currency, lpTotalNumberOfBytes As Currency, lpTotalNumberOfFreeBytes As Currency) As Long
Public Const FILE_ATTRIBUTE_DIRECTORY = &H10



Public Function FormatVBError(ByVal Number As Long, ByVal Description As String, ByVal Source As String) As String
    FormatVBError = "An error has occurred. Information below:" & vbCrLf & vbCrLf & _
               "Error Number: " & (Number And &H7FFFFFFF) & vbCrLf & _
               "Description :" & Description & vbCrLf & _
               "Source      :" & Source & vbCrLf & vbCrLf & _
               "You might try to [ignore] this error," & vbCrLf & _
               "but this may lead to even more obscure error messages."
               
End Function

Public Sub Center(f As Form)
    f.Move (Screen.Width - f.Width) / 2, (Screen.Height - f.Height) / 2
End Sub
Public Function FormatBytes(ByVal lSize As Currency) As String

Const MEGABYTE As Currency = &H100000
Const GIGABYTE  As Currency = &H40000000
Const KILOBYTE As Currency = 1024

If lSize > GIGABYTE Then
    FormatBytes = Format$((lSize / GIGABYTE), "#####.00") & " GB"
ElseIf lSize >= MEGABYTE Then
    FormatBytes = Format$((lSize / MEGABYTE), "####.00") & " MB"
ElseIf lSize >= KILOBYTE Then
    FormatBytes = Format$((lSize / KILOBYTE), "#####.00") & " KB"
Else
    FormatBytes = lSize & " Bytes"
End If
End Function
Public Function GetFreeSpace(ByVal strDisk As String) As Currency
    Dim currFreeBytes As Currency, currTotalBytes As Currency, currAvailableFreeBytes As Currency
    Dim lngRet As Long
    lngRet = GetDiskFreeSpaceEx(strDisk, currAvailableFreeBytes, currTotalBytes, currFreeBytes)
    If lngRet Then
        GetFreeSpace = currAvailableFreeBytes * 10000
    Else
        MsgBox ApiError(Err.LastDllError), vbOKOnly, vbCritical
    End If
End Function
Public Function TrimNull(ByVal strVal As String) As String
    Dim i As Long
    If Len(strVal) = 0 Then
        TrimNull = ""
        Exit Function
    End If
    i = InStr(strVal, vbNullChar)
    Select Case i
        Case 0, 1
            TrimNull = ""
        Case Else
            TrimNull = Left$(strVal, i - 1)
    End Select
End Function
Public Function FileExists(ByVal strFile As String) As Boolean
    Dim hFind As Long
    Dim WFD As WIN32_FIND_DATA
    hFind = FindFirstFile(strFile, WFD)
    If hFind = INVALID_HANDLE_VALUE Then
        FileExists = False
    Else
        FileExists = True
    End If
    FindClose hFind
        
End Function
Public Function FolderExists(ByVal sFolder As String) As Boolean
'Function to check if a folder exists

'Inputs:
'   sFolder: any legal path
'Returns:
'   true if path is present on system AND represents a folder.
    
    Dim hFind As Long
    Dim WFD As WIN32_FIND_DATA
    
    If sFolder Like "[a-zA-Z]:\" Then 'root folder.
        'FindFirstFile doesn't recognize the root folder as
        'a directory. We Need to do something else
        'Check for first (whatever) there! If that exists, so
        'does the root. Sneaky, huh?
        sFolder = sFolder & "*.*"
        hFind = FindFirstFile(sFolder, WFD)
        If hFind <> INVALID_HANDLE_VALUE Then
            FolderExists = True
        End If
        FindClose hFind
        Exit Function
    End If
    hFind = FindFirstFile(sFolder, WFD)
    If hFind = INVALID_HANDLE_VALUE Then
        FolderExists = False
        Exit Function
    Else
        If WFD.dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY Then
            'must be a folder
            FolderExists = True
        Else: FolderExists = False
        End If
        Call FindClose(hFind)
    End If
End Function


Public Function PathExists(ByVal strPath As String) As Boolean
Dim hFind As Long
Dim lngError As Long
Dim WFD As WIN32_FIND_DATA

Err.Clear
'look for anything in the intended folder
hFind = FindFirstFile(FixPath(strPath) & "*.*", WFD)

Select Case hFind
    Case INVALID_HANDLE_VALUE
        PathExists = False
    Case Else
        'This has been observed to work even for empty folders.
        'The reason being, the API returns entries for the parent
        'folder ".." and the current folder "."
        PathExists = True
End Select
FindClose hFind

End Function
Public Function CreateFolder(ByVal sFolder As String) As Boolean
'Code to attempt to create a folder
'or nest of folders in one fell swoop.
    
'NOTE: Does it the hard way, by creating one
'      directory level at a time, in succession.
'      There is a Windows API hack to accomplish this,
'      but it is undocumented, so is unsafe to use.
    
    Dim Start As Integer, Level As String, i As Integer
    Dim msg As String

    'Set up error trap
    On Error GoTo ErrMakeSubPaths

    'Elementary path check
    If Mid$(sFolder, 2, 2) <> ":\" And Mid$(sFolder, 1, 2) <> "\\" Then
        Debug.Print "Invalid Path? : " & sFolder
        'Let the error trap catch it
        MkDir sFolder
    End If

CreateDirs:
    Start = 4 'bypass root characters (I.E "C:\"
        
NextLevel:
        i = InStr(Start, sFolder, "\", 1)
        If i = 0 Then
            'At last level of path, so it's safe to create the
            'whole sFolder string now
            MkDir sFolder
            'Exit Function
        Else
            Level = Left$(sFolder, i - 1)
            MkDir Level
            Start = i + 1
            i = 0
            GoTo NextLevel
        End If
        
        'If We got here, we have succeeded
        If FolderExists(sFolder) Then
            CreateFolder = True
        End If
        
Done_CreateFolder:
    Exit Function

ErrMakeSubPaths:
    If Err = 75 Then
        'Trying to create a folder that already esists
        'Which is OK.
        Resume Next
    Else
        msg = "The Following Error Occurred Trying To Create The Folder" & vbCrLf _
        & sFolder & vbCrLf & vbCrLf _
        & Err & ": " & Error & vbCrLf & vbCrLf
        Err.Raise Err, "SorlingFunctions::CreateFolder", msg
    End If

End Function

Public Function FixPath(ByVal strPath As String, Optional ByVal strSep As String = "\") As String
    'Ensures that a backslash is at the end of the path
    If Right$(strPath, 1) = strSep Then
        FixPath = strPath
    Else
        FixPath = strPath & strSep
    End If
End Function
Public Function NoExtension(ByVal strPath As String) As String
    Dim i As Long
    i = InStrRev(strPath, ".")
    If i Then
        NoExtension = Left$(strPath, i - 1)
    Else
        NoExtension = strPath
    End If
End Function
Public Function NoFile(ByVal strPath As String, Optional ByVal strSep As String = "\") As String
    'Shaves off the filename (or the last atom of the path)
    Dim i As Long
    If Len(strPath) = 0 Then Exit Function
    'Locate last separator
    i = InStrRev(strPath, strSep)
    Select Case i
        'where was it?
        Case 0: NoFile = ""
        'Case 0: NoFile = strPath 'Nowhere. return whole string. ERROR!
        Case 3: NoFile = Left$(strPath, i) 'e.g "c:\"
        Case Else: NoFile = Left$(strPath, i - 1) 'shave off the path delimiter and everything after it.
    End Select
End Function
Public Function NoPath(ByVal strPath As String, Optional ByVal strSep As String = "\") As String
    Dim i As Long
    If Len(strPath) = 0 Then Exit Function
    i = InStrRev(strPath, strSep)
    If i Then
        NoPath = Mid$(strPath, i + 1)
    Else
        NoPath = strPath
    End If
End Function
Public Function Extension(ByVal strPath As String, Optional ByVal strPathSep As String = "\") As String
    Dim i As Long
    Dim strExt As String
    
    If Len(strPath) Then
        i = InStrRev(strPath, ".")
        If i Then
            strExt = Mid$(strPath, i + 1)
            'paranoid precaution: ensure that it really is the extension,
            'and not part of the path name
            i = InStr(strExt, strPathSep)
            If i = 0 Then
                'we're gold. Turn lowercase and return
                Extension = LCase$(strExt)
            End If
        End If
    End If
End Function

' Sort the items in array values() with bounds min and max.
Public Sub Quicksort(values() As String, _
              ByVal Min As Long, _
              ByVal Max As Long)

  Dim med_value As String
  Dim hi As Long
  Dim lo As Long
  Dim i As Long

  ' If the list has only 1 item, it's sorted.
  If Min >= Max Then Exit Sub

  ' Pick a dividing item randomly.
  i = Min + Int(Rnd(Max - Min + 1))
  med_value = values(i)

  ' Swap the dividing item to the front of the list.
  values(i) = values(Min)

  ' Separate the list into sublists.
  lo = Min
  hi = Max
  Do
    ' Look down from hi for a value < med_value.
    Do While values(hi) >= med_value
      hi = hi - 1
      If hi <= lo Then Exit Do
    Loop

    If hi <= lo Then
      ' The list is separated.
      values(lo) = med_value
      Exit Do
    End If

    ' Swap the lo and hi values.
    values(lo) = values(hi)

    ' Look up from lo for a value >= med_value.
    lo = lo + 1
    Do While values(lo) < med_value
      lo = lo + 1
      If lo >= hi Then Exit Do
    Loop

    If lo >= hi Then
      ' The list is separated.
      lo = hi
      values(hi) = med_value
      Exit Do
    End If

    ' Swap the lo and hi values.
    values(hi) = values(lo)
  Loop ' Loop until the list is separated.

  ' Recursively sort the sublists.
  Quicksort values, Min, lo - 1
  Quicksort values, lo + 1, Max

End Sub
Public Function ReverseInt(ByVal rhs As Integer) As Integer
    Dim bdata(0 To 1) As Byte
    CopyMemory bdata(0), rhs, 2
    ReverseInt = CInt(bdata(1)) + CInt(256) * CInt(bdata(0))
End Function
Public Function ReverseLong(ByVal rhs As Long) As Long
    Dim bdata(0 To 3) As Byte
    Dim lRes As Long
    CopyMemory bdata(0), rhs, 4
    lRes = bdata(3) + bdata(2) * 256 + bdata(1) * 256 ^ 2 + bdata(0) * 256 ^ 3
    ReverseLong = lRes
End Function

Function DumpBytes(ab() As Byte, cbBytes As Long) As String
    Dim i As Long, strRet As String
    For i = LBound(ab) To LBound(ab) + cbBytes - 1
        strRet = strRet & Right$("00" & Hex$(ab(i)), 2) & " "
    Next i
    DumpBytes = strRet
End Function
Function IsIDe() As Boolean
On Error GoTo ErrHandler
    Debug.Print 1 / 0
    Exit Function
ErrHandler:
    IsIDe = True
    Resume Next
End Function

Public Function ApiError(ByVal e As Long) As String

Dim s As String, c As Long
    s = String(MAX_PATH, 0)
    c = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM Or _
        FORMAT_MESSAGE_IGNORE_INSERTS, _
        ByVal 0&, e, 0&, s, Len(s), ByVal 0&)

    If c Then ApiError = Left$(s, c)

End Function
Public Function ByteArray_GetDword(b() As Byte, ByVal offset As Long) As Long
    CopyMemory ByteArray_GetDword, b(offset), 4
End Function
Public Function ByteArray_IsJPEG(b() As Byte) As Boolean
    Select Case ByteArray_GetDword(b(), 0)
        Case &HE0FFD8FF, &HE1FFD8FF
            ByteArray_IsJPEG = True
    End Select
End Function
Public Function ByteArray_IsGif(b() As Byte) As Boolean
    If ByteArray_GetDword(b(), 0) = &H38464947 Then
        ByteArray_IsGif = True
    End If
End Function
